function [ gw_is_table, gw_is_table_summary ] = make_gw_is_table(gwmerged,params)
% creates table of in-sample parameter estimates, stambaugh adjustments, 
% asymptotic and simulation-based standard errors for all the Goyal Welch
% (2008) variables in gwmerged
%
% params must contain 
%   nfutperiods = number of periods ahead to predict
%   nsim = number of simulations to run
%   lhsmultiplier = scaling for lhs
%   nlags = number of lags for Newey West

% this the table to populate
gw_is_table = []; 

% we always adjust for stambaugh bias in this table
params.adjuststambaugh = 1; 

disp([ 'Making Goyal Welch in-sample table for predicting next ' num2str(params.nfutperiods) ' months' ]); 
% loop through the sixteen gw variables
for i = 1:16
    % which x variable are we on?
    if( i == 1 ); x = gwmerged.dp/100; 
    elseif( i == 2); x = gwmerged.dy/100; 
    elseif( i == 3); x = gwmerged.ep/100; 
    elseif( i == 4); x = gwmerged.de/100; 
    elseif( i == 5); x = gwmerged.sigsq_rv; 
    elseif( i == 6); x = gwmerged.tbl; 
    elseif( i == 7); x = gwmerged.lty; 
    elseif( i == 8); x = gwmerged.ltr; 
    elseif( i == 9); x = gwmerged.tms; 
    elseif( i == 10); x = gwmerged.dfy; 
    elseif( i == 11); x = gwmerged.infl; 
    elseif( i == 12); x = gwmerged.bm/100; 
    elseif( i == 13); x = gwmerged.csp; 
    elseif( i == 14); x = gwmerged.ntis; 
    elseif( i == 15); x = gwmerged.lpayoutyield/100; 
    elseif( i == 16); x = gwmerged.cay; 
    end
    
    % limit attention to observations with available x
    thismerged = gwmerged(~isnan(x),:); 
    thisx = x(~isnan(x),:); 
    
    % call the function that computes the coefficients, adjusts for stambaugh bias, simulates SEs    
    in.r = thismerged.gwmkt; 
    in.x = thisx; 
    in.realvar = thismerged.rv_0; 
    in.predvar = thismerged.sigsq_rv; 
    [ols_stats, wls_stats ] = olswlsbootstrap_keepxsig(in,params); 
    
    % compute and tabulate everything

    % first for OLS
    ols_betahat = ols_stats.data_coeff(2);
    ols_betastambaugh = ols_stats.data_coeff(2)-ols_stats.stambaugh_bias;
    ols_se_nw = ols_stats.data_se_nw(2);    
    ols_pval_nw = 2*normcdf(-abs(ols_betastambaugh)/ols_se_nw); 
    ols_se_he_sim = ols_stats.std_he_sim_coeff(2);
    ols_pval_he_sim = ols_stats.pvalue2s_he(2); 
    
    olscol = [ ols_betastambaugh; ols_betahat; ...
        ols_se_nw; ols_pval_nw; ...
        ols_se_he_sim; ols_pval_he_sim ]; 
    
    % then for WLS
    wls_betahat = wls_stats.data_coeff(2);
    wls_betastambaugh = wls_stats.data_coeff(2)-wls_stats.stambaugh_bias;
    wls_se_nw = wls_stats.data_se_nw(2);    
    wls_pval_nw = 2*normcdf(-abs(wls_betastambaugh)/wls_se_nw); 
    wls_se_he_sim = wls_stats.std_he_sim_coeff(2);
    wls_pval_he_sim = wls_stats.pvalue2s_he(2); 
    
    wlscol = [ wls_betastambaugh; wls_betahat; ...
        wls_se_nw; wls_pval_nw; ...
        wls_se_he_sim; wls_pval_he_sim ]; 
    
    gw_is_table = [gw_is_table NaN(6,1) [ olscol wlscol ] ]; 
    
    disp([ 'Done with signal ' num2str(i) ' of 16' ]); 
end

% Compute summary stats for Table
olspvals = gw_is_table(6,2:3:end); 
wlspvals = gw_is_table(6,3:3:end); 
wlssimse_div_olssimse = gw_is_table(5,3:3:end)./gw_is_table(5,2:3:end); 

gw_is_table_summary = [ sum(olspvals <= 0.1) NaN sum(olspvals <= 0.05) NaN sum(olspvals <= 0.01) zeros(1,4); ...
    sum(wlspvals <= 0.1) NaN sum(wlspvals <= 0.05) NaN sum(wlspvals <= 0.01) NaN mean(wlssimse_div_olssimse) NaN sum( wlspvals < olspvals ) ]; 
