function [ oos_rsq_table, oos_rsq_table_summary, oos_dce_table, oos_dce_table_summary ] = ...
    make_gw_oos_table(gwmerged,params)
% creates table of out-of-sample R2 estimates using OLS/WLS, potentially
% paired with CT and PTV approaches
%
% params must contain 
%   nfutperiods = number of periods ahead to predict
%   lhsmultiplier = scaling for lhs
%   ntraining = number of training periods
%   ct_signs = predicted sign, required for Campbell Thompson
%   ptv_minsr = minimum fitted value for SR for PTV
%`  ptv_maxsr = maximum fitted value for SR for PTV

oos_rsq_table = []; 
oos_dce_table = []; 

% ptv bounds depend on nfutperiods
params.ptv_minsr = 0;
params.ptv_maxsr = sqrt(params.nfutperiods/12);

disp([ 'Making Goyal Welch out-of-sample tables for predicting next ' num2str(params.nfutperiods) ' months' ]); 

% only simulate return samples once to save time, use for all predictors
sim_rets = sim_returns(gwmerged.gwmkt,gwmerged.rv_0,params);

for i = 1:16
    fprintf('Working on predictor %d of 16...',i);
    % which x variable are we on?
    if( i == 1 ); x = gwmerged.dp/100; varname= 'dp'; 
    elseif( i == 2); x = gwmerged.dy/100; varname = 'dy'; 
    elseif( i == 3); x = gwmerged.ep/100; varname = 'ep';
    elseif( i == 4); x = gwmerged.de/100; varname = 'de';
    elseif( i == 5); x = gwmerged.rv_m11_0; varname = 'RV'; 
    elseif( i == 6); x = gwmerged.tbl; varname = 'tbl';
    elseif( i == 7); x = gwmerged.lty; varname = 'lty';
    elseif( i == 8); x = gwmerged.ltr; varname = 'ltr';
    elseif( i == 9); x = gwmerged.tms; varname = 'tms';
    elseif( i == 10); x = gwmerged.dfy; varname = 'dfy';
    elseif( i == 11); x = gwmerged.infl; varname = 'infl';
    elseif( i == 12); x = gwmerged.bm/100; varname = 'bm'; 
    elseif( i == 13); x = gwmerged.csp; varname = 'csp';
    elseif( i == 14); x = gwmerged.ntis; varname = 'ntis'; 
    elseif( i == 15); x = gwmerged.lpayoutyield/100; varname = 'lpy'; 
    elseif( i == 16); x = gwmerged.cay; varname = 'caya'; 
    end
    
    % limit attention to observations with available x
    thismerged = gwmerged(~isnan(x),:);
    thisx = x(~isnan(x),:); 
    thissim_rets = sim_rets(~isnan(x),:); 
    
    % predicted sign for campbell-thompson
    params.ct_sign = params.ct_signs(i); 
   
    % call the function that computes in- and out-of-sample R^2 on a
    % rolling basis, for Table 
    [ isrsq, oosrsq ] = oosisrsq(thismerged.gwmkt,thissim_rets,thisx,thismerged.sigsq, ...
        thismerged.rv_0,params); 
    
    % call the function that computes the change in certainty equivalent
    % from rolling OOS investing, for Internet Appendix only
    dce = oosdce(thismerged.mktrf,thismerged.rf,thisx,thismerged.sigsq, ...
        thismerged.rv_0,params); 
    
    olscol_rsq = [isrsq.ols; oosrsq.ols; oosrsq.ols_pvalue; oosrsq.ols_ct; oosrsq.ols_ct_pvalue; oosrsq.ols_ptv; oosrsq.ols_ptv_pvalue ]; 
    wlscol_rsq = [isrsq.wls; oosrsq.wls; oosrsq.wls_pvalue; oosrsq.wls_ct; oosrsq.wls_ct_pvalue; oosrsq.wls_ptv; oosrsq.wls_ptv_pvalue ]; 
    diff_rsq = [NaN; oosrsq.wls-oosrsq.ols; oosrsq.diff_pvalue; oosrsq.wls_ct - oosrsq.ols_ct; oosrsq.diff_ct_pvalue; ...
        oosrsq.wls_ptv - oosrsq.ols_ct; oosrsq.diff_ptv_pvalue ]; 
    
    olscol_dce = [ dce.ols; dce.ols_ct; dce.ols_ptv ];
    wlscol_dce = [ dce.wls; dce.wls_ct; dce.wls_ptv ];
    
    % Tabulate
    oos_rsq_table = [oos_rsq_table olscol_rsq wlscol_rsq diff_rsq]; 
    oos_dce_table = [oos_dce_table NaN(3,1) olscol_dce wlscol_dce ]; 

    % make figures
    make_oos_figures;
end

ols_is_rsq = oos_rsq_table(1,1:3:end); 
wls_is_rsq = oos_rsq_table(1,2:3:end); 
diff_is_rsq = wls_is_rsq - ols_is_rsq;

ols_oos_rsq = oos_rsq_table(2,1:3:end); 
wls_oos_rsq = oos_rsq_table(2,2:3:end);
diff_oos_rsq = wls_oos_rsq - ols_oos_rsq;

ols_pvals = oos_rsq_table(3,1:3:end);
wls_pvals = oos_rsq_table(3,2:3:end);
diff_pvals = oos_rsq_table(3,3:3:end); 

ols_ct_rsq = oos_rsq_table(4,1:3:end); 
wls_ct_rsq = oos_rsq_table(4,2:3:end);
diff_ct_rsq = wls_ct_rsq - ols_ct_rsq;

ols_ct_pvals = oos_rsq_table(5,1:3:end); 
wls_ct_pvals = oos_rsq_table(5,2:3:end);
diff_ct_pvals = oos_rsq_table(5,3:3:end); 

ols_ptv_rsq = oos_rsq_table(6,1:3:end); 
wls_ptv_rsq = oos_rsq_table(6,2:3:end);
diff_ptv_rsq = wls_ptv_rsq - ols_ptv_rsq;

ols_ptv_pvals = oos_rsq_table(7,1:3:end); 
wls_ptv_pvals = oos_rsq_table(7,2:3:end);
diff_ptv_pvals = oos_rsq_table(7,3:3:end);

oos_rsq_table_summary = [mean(ols_is_rsq) NaN ...
    mean(ols_oos_rsq) NaN sum(ols_oos_rsq > 0 ) NaN sum(ols_pvals < 0.1) NaN sum(ols_pvals < 0.05) NaN sum(ols_pvals < 0.01) NaN ... 
    mean(ols_ct_rsq) NaN sum(ols_ct_rsq > 0 ) NaN sum(ols_ct_pvals < 0.1) NaN sum(ols_ct_pvals < 0.05) NaN sum(ols_ct_pvals < 0.01) NaN ... 
    mean(ols_ptv_rsq) NaN sum(ols_ptv_rsq > 0 ) NaN sum(ols_ptv_pvals < 0.1) NaN sum(ols_ptv_pvals < 0.05) NaN sum(ols_ptv_pvals < 0.01); ... 
    mean(wls_is_rsq) NaN ...
    mean(wls_oos_rsq) NaN sum(wls_oos_rsq > 0 ) NaN sum(wls_pvals < 0.1) NaN sum(wls_pvals < 0.05) NaN sum(wls_pvals < 0.01) NaN ... 
    mean(wls_ct_rsq) NaN sum(wls_ct_rsq > 0 ) NaN sum(wls_ct_pvals < 0.1) NaN sum(wls_ct_pvals < 0.05) NaN sum(wls_ct_pvals < 0.01) NaN ... 
    mean(wls_ptv_rsq) NaN sum(wls_ptv_rsq > 0 ) NaN sum(wls_ptv_pvals < 0.1) NaN sum(wls_ptv_pvals < 0.05) NaN sum(wls_ptv_pvals < 0.01); ... 
    mean(diff_is_rsq) NaN ...
    mean(diff_oos_rsq) NaN sum(diff_oos_rsq > 0 ) NaN sum(diff_pvals < 0.1) NaN sum(diff_pvals < 0.05) NaN sum(diff_pvals < 0.01) NaN ... 
    mean(diff_ct_rsq) NaN sum(diff_ct_rsq > 0 ) NaN sum(diff_ct_pvals < 0.1) NaN sum(diff_ct_pvals < 0.05) NaN sum(diff_ct_pvals < 0.01) NaN ... 
    mean(diff_ptv_rsq) NaN sum(diff_ptv_rsq > 0 ) NaN sum(diff_ptv_pvals < 0.1) NaN sum(diff_ptv_pvals < 0.05) NaN sum(diff_ptv_pvals < 0.01) ... 
    ]; 

% For output purposes do not need isrsq
oos_rsq_table = oos_rsq_table(2:end,:); 

ols_dce = oos_dce_table(1,2:3:end); 
wls_dce = oos_dce_table(1,3:3:end); 
ols_ctdce = oos_dce_table(2,2:3:end); 
wls_ctdce = oos_dce_table(2,3:3:end); 
ols_ptvdce = oos_dce_table(3,2:3:end); 
wls_ptvdce = oos_dce_table(3,3:3:end); 

oos_dce_table_summary = [ mean(ols_dce) NaN sum(ols_dce > 0) NaN NaN ... 
    NaN mean(ols_ctdce) NaN sum(ols_ctdce >0) NaN NaN ...
    NaN mean(ols_ptvdce) NaN sum(ols_ptvdce>0) NaN NaN ; ...
    mean(wls_dce) NaN sum(wls_dce > 0) NaN sum(wls_dce > ols_dce) ...
    NaN mean(wls_ctdce) NaN sum(wls_ctdce >0) NaN sum(wls_ctdce > ols_ctdce) ...
    NaN mean(wls_ptvdce) NaN sum(wls_ptvdce >0) NaN sum(wls_ptvdce > ols_ptvdce)]; 