function out = make_hodvsnw_table(in,params)
% function out = make_hodvsnw_table(in,params)
% in must have: x, r, hod_sigsq (the var forecast I use for main analysis),
%               rv (realized variance within period), and var_predictors
%               (what we forecast with)
% params must have: nsim, lhsmultiplier, nfutperiods, adjuststambaugh,
% hod_nlags, nw_nlags

T = length(in.r);

% step 1: simulate returns
truevar = in.rv; 
simr_he = sim_returns( in.r, truevar, params); 

% step 2: hodrick approach. standard from rest of paper
rhs = ones(T-params.nfutperiods,1); 
rollx = conv(in.x,ones(1,params.nfutperiods),'valid'); 
rollx = rollx(~isnan(rollx),1); 
rollx = rollx*var(in.x)/var(rollx);

rhs = [ rhs rollx(1:end-1,1) ];

% predvar = used for weighting regressions
predvar_hod = in.hod_sigsq; 

% data coeff
lhs_data = params.lhsmultiplier*in.r(1+params.nfutperiods:end,1); 
[ data_ols_coeff_hod, data_ols_se_nw_hod ] = olsgmm(lhs_data,rhs,params.hod_nlags,1); 
[ data_wls_coeff_hod, data_wls_se_nw_hod ] = wlsev(lhs_data,rhs,predvar_hod(params.nfutperiods:end-1,:),params.hod_nlags,1); 

% sim coeff
lhs_sim = params.lhsmultiplier*simr_he(params.nfutperiods+1:end,:); 

% ols
coeff = olsgmm(lhs_sim,rhs,0,-1); 
sim_ols_coeff_hod = coeff';
    
% wls
coeff = wlsev(lhs_sim,rhs,predvar_hod(params.nfutperiods:end-1,1),0,-1); 
sim_wls_coeff_hod = coeff';

% now adjust for stambaugh bias using bootstrap of all three variables
if(params.adjuststambaugh)

disp('Adjusting for Stambaugh bias'); 
    stambaugh_bias = bootstrap_stambaugh(in.r,in.x,params); 
else
    stambaugh_bias = 0;
end

% make hod cols
ols_hod_col = [ data_ols_coeff_hod(2)-stambaugh_bias; data_ols_coeff_hod(2); ...
    data_ols_se_nw_hod(2); 2*normcdf(-abs(data_ols_coeff_hod(2)-stambaugh_bias)/data_ols_se_nw_hod(2)); ...
    std(sim_ols_coeff_hod(:,2)); mean( abs(sim_ols_coeff_hod(:,2)) > (data_ols_coeff_hod(2)-stambaugh_bias)) ]; 
wls_hod_col = [ data_wls_coeff_hod(2)-stambaugh_bias; data_wls_coeff_hod(2); ...
    data_wls_se_nw_hod(2); 2*normcdf(-abs(data_wls_coeff_hod(2)-stambaugh_bias)/data_wls_se_nw_hod(2)); ...
    std(sim_wls_coeff_hod(:,2)); mean( abs(sim_wls_coeff_hod(:,2)) > (data_wls_coeff_hod(2)-stambaugh_bias)) ]; 

% step 3: pure overlap approach with nw correction. for oapp only
rhs = [ones(T-params.nfutperiods,1) in.x(1:end-params.nfutperiods,1)]; 
lhs_data = params.lhsmultiplier*conv(in.r(2:end,1),ones(1,params.nfutperiods),'valid'); 
csr = cumsum(simr_he,1); 
lhs_sim = params.lhsmultiplier*( csr(1+params.nfutperiods:end,:) - csr(1:end-params.nfutperiods,:) ); 

% compute predvar in the data
% try to predict sum of realized variance in nfutperiods
lhs_firststage = conv(in.rv(2:end,1),ones(1,params.nfutperiods),'valid'); 
rhs_firststage = [ ones(T-params.nfutperiods,1) in.var_predictors(1:end-params.nfutperiods,:) ]; 

coeff_firststage  = rhs_firststage \ lhs_firststage; 
predvar_nw = [ ones(T,1) in.var_predictors ]*coeff_firststage; 

% data coeff
[ data_ols_coeff_nw, data_ols_se_nw_nw ] = olsgmm(lhs_data,rhs,params.nw_nlags,1); 
[ data_wls_coeff_nw, data_wls_se_nw_nw ] = wlsev(lhs_data,rhs,predvar_nw(1:end-params.nfutperiods,:),params.nw_nlags,1); 

% sim coeff
sim_ols_coeff_nw = olsgmm(lhs_sim,rhs,0,-1)'; 
sim_wls_coeff_nw = wlsev(lhs_sim,rhs,predvar_nw(1:end-params.nfutperiods,:),0,-1)'; 

% make nw cols
ols_nw_col = [ data_ols_coeff_nw(2)-stambaugh_bias; data_ols_coeff_nw(2); ...
    data_ols_se_nw_nw(2); 2*normcdf(-abs(data_ols_coeff_nw(2)-stambaugh_bias)/data_ols_se_nw_hod(2)); ...
    std(sim_ols_coeff_nw(:,2)); mean( abs(sim_ols_coeff_nw(:,2)) > (data_ols_coeff_nw(2)-stambaugh_bias)) ]; 
wls_nw_col = [ data_wls_coeff_nw(2)-stambaugh_bias; data_wls_coeff_nw(2); ...
    data_wls_se_nw_nw(2); 2*normcdf(-abs(data_wls_coeff_nw(2)-stambaugh_bias)/data_wls_se_nw_nw(2)); ...
    std(sim_wls_coeff_nw(:,2)); mean( abs(sim_wls_coeff_nw(:,2)) > (data_wls_coeff_nw(2)-stambaugh_bias)) ]; 

out = [ ols_hod_col ols_nw_col NaN(6,1) wls_hod_col wls_nw_col ]; 
