function [ out_panels, critvalues ] = make_sim_table(in,params)
% function [ out_panels, critvalues ] = make_sim_table(in,params)
% in must be a table containing columns r, x, predvar, realvar
% params must have the following fields:
%   nsim
%   truebeta
%   crittstat = critical t-stat for table
%   critbeta = beta has sim. p value = % > critbeta
%   critpval = critical p-value to compute critbetas
%   nfutperiods = number of future return periods to predict
%   nlags = number of lags for Newey West
% 
% remove parts of the in table with nans

% remove nans
selector = ~isnan(in.r) & ~isnan(in.x) & ~isnan(in.predvar) & ~isnan(in.realvar) ;
in.r = in.r(selector,1); in.x = in.x(selector,:); in.predvar(selector,1); in.realvar(selector,1); 

T = length(in.r); 

if( params.sim_x ) % sim x
%     try simming x and r, keeping predvar and true var as observed.
%   1) Simulate returns according to r_t = mu_r+eps_t * sqrt(truevar_t),
%   eps_t resampled
%   2) Simulate x according to x_t - mu_x = rho_x*(x_{t-1}-mu_x) + delta_t,
%   where delta_t = psi*(r_t-mu_r) + delta_orth_t, psi estimated from data,
%   eps_t from resampling, delta_orth_t from resampling independently
%   3) Add truebeta*x_t to returns
    % estimate mu_x and rho_x
    
    % get our data eps_t
    % predvar = whatever you want to use for weighting regressions
    predvar = in.predvar;     
    
    % truevar = whatever you want to use as the true variance for wild bootstrap
    truevar = in.realvar; 
    
    % Estimate return residuals for simulations
    mu_r = mean(in.r ./ sqrt(truevar)) / mean(1 ./ sqrt(truevar));
    data_eps = (in.r - mu_r) ./ sqrt(truevar);
    data_eps = data_eps(2:end,1); 
    data_eps = data_eps / std(data_eps); 
    
    % get our data_psi and data_delta_orth
    mu_x = mean(in.x); 
    rho_x = autocorr(in.x); 
     
    % compute data eps
    data_delta = in.x(2:end) - mu_x - rho_x*(in.x(1:end-1,1)-mu_x); 
     
    % regress on demeaned return to get psi
    data_psi = (in.r(2:end) - mean(in.r(2:end))) \ data_delta; 
     
    % now get data delta orth
    data_delta_orth = data_delta - data_psi*(in.r(2:end)-mean(in.r(2:end)));  
     
    % can do simulations now
    coeff_ols = NaN(2,params.nsim);
    SE_ols = NaN(2,params.nsim); 
    
    coeff_wls = NaN(2,params.nsim);
    SE_wls = NaN(2,params.nsim);

    simr_he_all = NaN(length(in.r),params.nsim); 
    
    fprintf('        '); 
    for simi = 1:params.nsim
        % Redraw errors
        sim_eps = data_eps(randsample(T-1,T,true)); 
        sim_delta_orth = data_delta_orth(randsample(T-1,T-1,true)); 
        
        % use sim_eps to compute simulated returns
        simr_he = mu_r + sqrt(truevar).*sim_eps ;    
        
        % This will be initial x_t
        init_x = in.x(randi(T),:);

        % Compute x and rhs
        csxerrors = cumsum(sim_delta_orth./(rho_x.^(-T/2:T/2-2)')); 
        simx = [init_x; mu_x + (init_x-mu_x)*rho_x.^((1:T-1)') + ...
            csxerrors.*rho_x.^((-T/2:T/2-2)')]; 
        
        % Add in the part due to returns
        csrets = cumsum(  (simr_he(2:end) - mean(simr_he(2:end))) ./(rho_x.^(-T/2:T/2-2)') ); 
        simx_retpart = [ 0; data_psi*csrets.*rho_x.^((-T/2:T/2-2)') ]; 
        
        simx = simx + simx_retpart; 
        
        if( sum(isinf(simx)) > 0 )
            simx(1,1) = init_x; 
            for j = 2:T
                simx(j,1) = mu_x + rho_x*(simx(j-1,1)-mu_x) + sim_delta_orth(j-1,1) + ...
                    (simr_he(j) - mean(simr_he))*psi; 
            end
        end        
        
        % now have our x, use rolling version to adjust for overlap
        rollx = conv(simx,ones(1,params.nfutperiods),'valid'); 
        rollx = rollx(~isnan(rollx),1); 
        rollx = rollx*var(simx)/var(rollx);

        rhs_sim = [ ones(T-params.nfutperiods,1) rollx(1:end-1,1) ];               

        % finally, add in true coeff to rets
        simr_he = simr_he + [zeros(params.nfutperiods,1); rhs_sim(:,2)]*params.truebeta; 
        
        lhs_sim = params.lhsmultiplier*simr_he(params.nfutperiods+1:end,:);
        
        % ols 
        [ coeff_ols(:,simi), SE_ols(:,simi) ] = olsgmm(lhs_sim,rhs_sim,params.nlags,1); 

        % wls 
        [ coeff_wls(:,simi), SE_wls(:,simi) ] = wlsev(lhs_sim,rhs_sim,predvar(params.nfutperiods:end-1,1),params.nlags,1); 
        
        simr_he_all(:,simi) = simr_he; 
        
        if( mod(simi,500) == 0 && floor(100*simi/params.nsim) < 100 )
            fprintf(1,'\b\b\b\b\b\b\b\b%2d%% done',floor(100*simi/params.nsim));
        end
    end
    fprintf(1,'\b\b\b\b\b\b\b\b%2d%% done',floor(100*simi/params.nsim));
    fprintf('\n');
    
else % sim r only
    % Redo x so we are estimating the t+1,t+nfutperiods overlapping
    % predicability
    rollx = conv(in.x,ones(1,params.nfutperiods),'valid'); 
    rollx = rollx(~isnan(rollx),1); 
    rollx = rollx*var(in.x)/var(rollx);

    rhs = [ ones(T-params.nfutperiods,1) rollx(1:end-1,1) ];

    % truevar = whatever you want to use as the true variance for wild bootstrap
    truevar = in.realvar; 

    % predvar = whatever you want to use for weighting regressions
    predvar = in.predvar; 

    % Estimate return residuals for simulations
    mu_r = mean(in.r ./ sqrt(truevar)) / mean(1 ./ sqrt(truevar));
    data_eps = (in.r - mu_r) ./ sqrt(truevar);

    data_eps = data_eps / std(data_eps); 

    sim_eps_col = data_eps(randi(T-1,[T-params.nfutperiods, params.nsim]),1); 
    sim_eps_mat = reshape(sim_eps_col,T-params.nfutperiods,params.nsim); 
    simr_he = mu_r + ([zeros(params.nfutperiods,1); rhs(:,2:end)]*params.truebeta)*ones(1,params.nsim) + ...
        [zeros(params.nfutperiods,params.nsim); (sqrt(truevar(params.nfutperiods+1:end,1))*ones(1,params.nsim)).*sim_eps_mat ];   

    % Now do regressions with heteroskedastic simr
    lhs_sim = params.lhsmultiplier*simr_he(params.nfutperiods+1:end,:); 

    % ols 
    [ coeff_ols, SE_ols ] = olsgmm(lhs_sim,rhs,params.nlags,1); 

    % wls 
    [ coeff_wls, SE_wls ] = wlsev(lhs_sim,rhs,predvar(params.nfutperiods:end-1,1),params.nlags,1); 
    
    simr_he_all = simr_he; 
end

if( params.do_oos )
    % get OOS R2 across different sims
    oosparams.ct_sign = 1; oosparams.ptv_minsr = 0; oosparams.ptv_maxsr = 1; % none of this matters here since I don't use ptv or ct for sim tables
    oosparams.nfutperiods = params.nfutperiods;
    oosparams.ntraining = params.ntraining; 
    simr_he_oos = simr_he_all(:,floor(1:end/4)); 
    [~, oosrsq] = oosisrsq(in.r,simr_he_oos,in.x,in.sigsq,in.realvar,oosparams); 
end

sim_coeff = [ coeff_ols(2,:)' coeff_wls(2,:)' ];
sim_asySE = [ SE_ols(2,:)' SE_wls(2,:)' ]; 

% quantiles of |b| to output
quants = 1-params.critpvalues; 

% compute a bunch of stuff for the tables
tstats = sim_coeff ./ sim_asySE; 
pvals = 2*(1-normcdf(abs(tstats))); 

means = mean(sim_coeff);
medians = median(sim_coeff);
abs_percentiles = quantile(abs(sim_coeff),quants); 
meanasySE = mean(sim_asySE); 
asy_pvals_low = [ mean( abs(tstats(:,1)) >= norminv(1-0.5*params.critpvalues) )' ... 
    mean( abs(tstats(:,2)) >= norminv(1-0.5*params.critpvalues) )' ];
simSE = std(sim_coeff); 
probbetasigpos = [ mean(sim_coeff(:,1) >= params.critbeta_ols)' ...
    mean(sim_coeff(:,2) >= params.critbeta_wls)' ]; 

ols_subsample = pvals(:,1) < 0.05; 
means_olssubsample = mean(abs(sim_coeff(ols_subsample,:))); 
probs_olssubsample = mean(pvals(ols_subsample,:) < 0.05);

wls_subsample = pvals(:,2) < 0.05; 
means_wlssubsample = mean(abs(sim_coeff(wls_subsample,:))); 
probs_wlssubsample = mean(pvals(wls_subsample,:) < 0.05);

means_fullsample = mean(abs(sim_coeff));
probs_fullsample = mean(pvals < 0.05);

if( params.do_oos )
    % OOS stats
    oosrsq_means = [ mean(oosrsq.ols_sim) mean(oosrsq.wls_sim) ]; 
    oosrsq_stdevs = [ std(oosrsq.ols_sim) std(oosrsq.wls_sim) ]; 
    oosrsq_medians = [  median(oosrsq.ols_sim) median(oosrsq.wls_sim) ]; 
    oosrsq_pos = [ mean(oosrsq.ols_sim > 0) mean(oosrsq.wls_sim > 0) ];
    oosrsq_percentiles = quantile([oosrsq.ols_sim' oosrsq.wls_sim'],quants);
    oosrsq_simsigpos = [ mean(oosrsq.ols_sim' > params.critoosrsq_ols)' mean(oosrsq.wls_sim' > params.critoosrsq_wls)' ]; 
else
    oosrsq_means = NaN(1,2); 
    oosrsq_stdevs = NaN(1,2); 
    oosrsq_medians = NaN(1,2); 
    oosrsq_pos = NaN(1,2); 
    oosrsq_percentiles = NaN(length(quants),2); 
    oosrsq_simsigpos = NaN(length(quants),2); 
end

% Panel A: no predictability null, include two sided stuff at bottom
out_panels.PanelA = [ means; medians; simSE; meanasySE; asy_pvals_low; abs_percentiles; ...
    oosrsq_means; oosrsq_stdevs; oosrsq_pos; oosrsq_percentiles ];

% Panel B: predictability null, include one sided stuff at bottom 
out_panels.PanelB = [ means; medians; simSE; meanasySE; probbetasigpos;  ...
    oosrsq_means; oosrsq_simsigpos];

% Panel C: data mining stuff
out_panels.PanelC = [ means_fullsample; probs_fullsample; ...
                    NaN(1,2); 
                    means_olssubsample; probs_olssubsample; ...
                    NaN(1,2); 
                    means_wlssubsample; probs_wlssubsample ]; 


critvalues.critbeta_ols = quantile(sim_coeff(:,1),1-0.5*params.critpvalues);  % these are two-sided
critvalues.critbeta_wls = quantile(sim_coeff(:,2),1-0.5*params.critpvalues);  % these are two-sided
if( params.do_oos )
    critvalues.critoosrsq_ols = quantile(oosrsq.ols_sim,1-0.5*params.critpvalues); 
    critvalues.critoosrsq_wls = quantile(oosrsq.wls_sim,1-0.5*params.critpvalues); 
else
    critvalues.critoosrsq_ols = NaN; 
    critvalues.critoosrsq_wls = NaN; 
end
