function [ ols_stats, wls_stats, ols_simcoeff, wls_simcoeff ] = olswlsbootstrap_keepxsig(in,params)
% function [ ols_stats, wls_stats, ols_simcoeff, wls_simcoeff ] = olswlsbootstrap_keepxsig(in,params)
% in must be a table containing columns: r, x, predvar, realvar
%
% r is a log return
% x is a predictor (can be multivariate, not including constant)
% predvar is prediction for var(r_t+1), computed elsewhere
% realvar is realized variance within period t
%
% r, x, predvar, realvar all observed  at same time, so regression is
% sum(r_{t+1,t+params.nfutperiods}) = a + b*x_t, potentially weighted using
% predvar
%
% params must have the following fields:
%   nsim = number of simulations to do
%   nfutperiods = number of future return periods to predict
%   adjuststambaugh = boolean for whether to do stambaugh bias correction
%   lhsmultiplier = multiplier to hit LHS with
%   nlags = number of lags for Newey West (optional, otherwise default to
%   Newey West (1994) lag selection
%
% "Wild bootstrap" that keeps x, predvar as observed, redraws r
% heteroskedastically. Simulations use realvar as "true" variance
% generating the data, predvar for WLS weights

% remove nans
selector = ~isnan(in.r) & ~isnan(in.x) & ~isnan(in.predvar) & ~isnan(in.realvar) ;
in.r = in.r(selector,1); in.x = in.x(selector,:); in.predvar(selector,1); in.realvar(selector,1); 

T = length(in.r); 
K = size(in.x,2)+1; 

if( isfield(params,'nlags') )
    nlags = params.nlags; 
else
    nlags = floor(12*(T/100)^(2/9)); 
end

% Use Hodrick (1992) mappring to equivalent non-overlapping regressions
rhs = ones(T-params.nfutperiods,1);
for xi = 1:K-1
    rollx = conv(in.x(:,xi),ones(1,params.nfutperiods),'valid'); 
    rollx = rollx(~isnan(rollx),1); 
    rollx = rollx*var(in.x)/var(rollx);

    rhs = [ rhs rollx(1:end-1,1) ];
end

% truevar = true variance for wild bootstrap
truevar = in.realvar; 

% predvar = used for weighting regressions
predvar = in.predvar; 

% Generate simulated returns   
simr_he = sim_returns( in.r, truevar, params); 

% Now do regressions with heteroskedastic simr
lhs_sim = params.lhsmultiplier*simr_he(params.nfutperiods+1:end,:); 

% ols in sims
coeff = olsgmm(lhs_sim,rhs,0,-1); 
sim_ols_coeff_he = coeff';
    
% wls in sims
coeff = wlsev(lhs_sim,rhs,predvar(params.nfutperiods:end-1,1),0,-1); 
sim_wls_coeff_he = coeff';

data_lhs = params.lhsmultiplier*in.r(1+params.nfutperiods:end,1); 
data_rhs = rhs;

[ data_ols_coeff, data_ols_se_nw, ~, R2adj_ols ] = olsgmm(data_lhs,data_rhs,nlags,1); 
[ data_wls_coeff, data_wls_se_nw, ~, R2adj_wls ] = wlsev(data_lhs,data_rhs,predvar(params.nfutperiods:end-1,:),nlags,1); 

% now adjust for stambaugh bias using bootstrap of all three variables
if(params.adjuststambaugh)

disp('Adjusting for Stambaugh bias'); 
    stambaugh_bias = bootstrap_stambaugh(in.r,in.x,params); 
else
    stambaugh_bias = 0;
end

ols_stats.stambaugh_bias = stambaugh_bias;
wls_stats.stambaugh_bias = stambaugh_bias; 

% tabulate ols_stats for output
ols_stats.data_coeff = data_ols_coeff'; 
ols_stats.data_se_nw = data_ols_se_nw'; 
ols_stats.mean_he_sim_coeff = mean(sim_ols_coeff_he); 
ols_stats.std_he_sim_coeff = std(sim_ols_coeff_he); 
ols_stats.R2 = R2adj_ols;

% one-sided pvalues
if( data_ols_coeff(1,1) > 0 )
    ols_stats.pvalue1s_he(1,1) = mean(sim_ols_coeff_he(:,1) > data_ols_coeff(1,1));
else
    ols_stats.pvalue1s_he(1,1) = mean(sim_ols_coeff_he(:,1) < data_ols_coeff(1,1)); 
end
if( data_ols_coeff(2,1)-stambaugh_bias > 0 )
    ols_stats.pvalue1s_he(1,2) = mean(sim_ols_coeff_he(:,2) > data_ols_coeff(2,1)-stambaugh_bias);
else
    ols_stats.pvalue1s_he(1,2) = mean(sim_ols_coeff_he(:,2) < data_ols_coeff(2,1)-stambaugh_bias); 
end

% two-sided pvalues
ols_stats.pvalue2s_he = [ mean(abs(sim_ols_coeff_he(:,1)) > abs(data_ols_coeff(1,1))) ...
    mean(abs(sim_ols_coeff_he(:,2)) > abs(data_ols_coeff(2,1)-stambaugh_bias)) ] ;

% Now WLS outputs
wls_stats.data_coeff = data_wls_coeff'; 
wls_stats.data_se_nw = data_wls_se_nw'; 
wls_stats.mean_he_sim_coeff = mean(sim_wls_coeff_he); 
wls_stats.std_he_sim_coeff = std(sim_wls_coeff_he); 
wls_stats.R2 = R2adj_wls; 

% one-sided pvalues
if( data_wls_coeff(1,1) > 0 )
    wls_stats.pvalue1s_he(1,1) = mean(sim_wls_coeff_he(:,1) > data_wls_coeff(1,1));
else
    wls_stats.pvalue1s_he(1,1) = mean(sim_wls_coeff_he(:,1) < data_wls_coeff(1,1)); 
end
if( data_wls_coeff(2,1) - stambaugh_bias > 0 )
    wls_stats.pvalue1s_he(1,2) = mean(sim_wls_coeff_he(:,2) > data_wls_coeff(2,1)- stambaugh_bias);
else
    wls_stats.pvalue1s_he(1,2) = mean(sim_wls_coeff_he(:,2) < data_wls_coeff(2,1)- stambaugh_bias); 
end

% two-sided pvalues
wls_stats.pvalue2s_he = [ mean(abs(sim_wls_coeff_he(:,1)) > abs(data_wls_coeff(1,1))) ...
    mean(abs(sim_wls_coeff_he(:,2)) > abs(data_wls_coeff(2,1)-stambaugh_bias)) ] ;

ols_simcoeff = sim_ols_coeff_he; 
wls_simcoeff = sim_wls_coeff_he; 

end
